require('dotenv').config({ path: './.env' })

const { log } = require('./util/log')
const config = require('./config.json')
const db = require('./util/db')
const Campanha = require('./model/Campanha')
const ContatoNumero = require('./model/CampanhaContatoNumeros')
const ContatoCampo = require('./model/CampanhaContatoCampo')
const Atendimento = require('./model/ClienteAtendimento')
const Cliente = require('./model/Cliente')
const Fila = require('./model/Fila')
const Layout = require('./model/Layouts')
const LayoutsCampos = require('./model/LayoutsCampos')
const upgrade = require('./util/db_upgrade')

const queue_type = new Map()
//queue_type.set(3, "whatsApi")
//queue_type.set(5, "zenvia")
//queue_type.set(6, "infobip")
queue_type.set(7, require('./util/gupshup'))

function sleep(ms) {
    return new Promise((resolve) => {
        setTimeout(resolve, ms);
    });
}

async function runCampanhas(ids) {
    const campanhas = await Campanha.query
        .where(Campanha.where.in('id', ids))
        .innerJoin(Fila, { fields: ['name', 'queue_type', 'emailConfig'], first: true })
        .innerJoin(Layout.query.innerJoin(LayoutsCampos, { orderBy: 'posicao_csv' }))
        .fetch()

    for (const campanha of campanhas) {
        log(`Executando campanha id: ${campanha.id}, Nome: ${campanha.nome}`)

        if (campanha.tipo_campanha === 1) {
            if (queue_type.has(campanha.fila.queue_type)) {
                const csa = queue_type.get(campanha.fila.queue_type)
                csa.start()

                campanha.csa = csa
                campanha.MensagensPorSeg = csa.mensagens_por_segundo
                log('Aguardando CSA conectar')
                await sleep(5000)
                log('Cancei de esperar, vou meter...')

                // Se tem a coluna rota tenta dar parse
                if (campanha.rota) {
                    try {
                        campanha.rota = JSON.parse(campanha.rota)
                    } catch (error) {
                        campanha.rota = campanha.rota
                    }
                }
                campanha.layoutsCampos = (campanha.layouts && campanha.layouts[0].layoutsCampos) ? JSON.parse(JSON.stringify(campanha.layouts[0].layoutsCampos)) : undefined
                delete campanha.layouts

                if (campanha.layoutsCampos) {
                    const fnome = campanha.layoutsCampos.find(c => c.xcontact === 'nome')
                    campanha.campo_nome = fnome ? fnome.nome : undefined
                }

                campanha.limit = 500
                campanha.intervalo = 1000 / campanha.MensagensPorSeg
                campanha.load = false

                // Cria uma função que fica tentando buscar novos números para discar a cada 30s
                // Util para campanhas com poucos números pois o processamento da lista retornada pelo load é mais rápida que o retorno de envio 
                campanha.retry = setInterval(() => {
                    log(`Retry, Campanha id: ${campanha.id}, Nome: ${campanha.nome}, no momento ela está load = ${campanha.load}`)
                    if (!campanha.load) loadNumeros(campanha)
                }, 30 * 1000)

                loadNumeros(campanha)
            }
            else {
                log(`[ERRO] [Tipo de FILA não suportado] A fila ${campanha.fila.name} que está associada a campanha de id = ${campanha.id} é do tipo = ${campanha.fila.queue_type}. Esse tipo NÃO é suportado, apenas filas de X5Api, Zenvia, infobipe e Gup são suportadas.`)
            }
        }
        else {
            log(`[ERRO] [Tipo não suportado] - Acampanha de id = ${campanha.id} é de tipo = ${campanha.tipo_campanha}. Apenas campanhas de WhatsApp são suportadas.`)
        }
    }
}

async function loadNumeros(campanha) {
    // if(!isConetado) return log('Deconectado do socket, NAO VAI CARREGAR NUMEROS')

    campanha.load = true
    try {
        const ret = await db.query(`
        with updated as (
            UPDATE public.campanhas_contato_numeros 
            set tentativas = tentativas + 1, processando = true
            WHERE id_campanha = $1 AND id IN (
                SELECT id 
                FROM public.campanhas_contato_numeros 
                WHERE id_campanha = $1
                AND processando = false
                AND sucesso = false 
                AND tentativas < 3
                ORDER BY fone_sequencia ASC, tentativas ASC, prioridade DESC, id_contato ASC
                LIMIT $2) 
            returning *
        )
        select a.*, b.campos
        from updated a,
        (SELECT bb.id_contato, json_object_agg(bb.nome, bb.valor) campos FROM campanhas_contato_campos bb GROUP by bb.id_contato) b
        WHERE a.id_contato = b.id_contato
        ORDER BY prioridade DESC,fone_sequencia ASC,tentativas ASC,id_contato ASC;
        `, [campanha.id, campanha.limit])
        if (ret && ret.rows && ret.rows.length > 0) {
            campanha.numeros = ret.rows
            log(`Iniciando envio de ${campanha.numeros.length} mensagens a uma taxa de 1 mensagem/${campanha.intervalo}ms`)
            AgendaDiscar(campanha)
        }
        else {
            log(`[AVISO-LOADNUMEROS] - Não há mais numeros para discar para campanha id: ${campanha.id}, Nome: ${campanha.nome}`)
            campanha.load = false
        }
    } catch (error) {
        log(`[ERRO-LOADNUMEROS] - Erro ao carregar números para campanha id: ${campanha.id}, Nome: ${campanha.nome}, ERRO: `, error)
        campanha.load = false
    }
}

async function AgendaDiscar(campanha) {

    campanha.timer = setInterval(() => {
        if (campanha.numeros.length > 0) {
            const rem = campanha.numeros.splice(0, 1)
            const contato = rem[0]
            Discar(contato, campanha)
        }
        else {
            clearInterval(campanha.timer)
            log(`Buscando mais ${campanha.limit} mumeros para campanha de id = ${campanha.id}`)
            loadNumeros(campanha)
        }
    }, campanha.intervalo);
}

async function Discar(contato, campanha) {
    log(`Enviando para: ${contato.numero} tentativa: ${contato.tentativas}`)
    const ret = await campanha.csa.enviaComTemplate(campanha, contato)
    log(`O envio para o numero: ${contato.numero} do contato: ${contato.id} retornou sucesso: ${!ret.error}`)
    new ContatoNumero({ id: contato.id, processando: false, sucesso: !ret.error }).save()
    if (!ret.error && config.atendimento.criar) {
        log(`Vamos criar o atendimento para o contato id = ${contato.id}, numero: ${contato.numero}`)
        criarAtendimento(campanha, contato)
    }
    // // Simulando random delay para envio de cada msg
    // const randomdelay = Math.floor(Math.random() * (20000 - 1000 + 1)) + 1000
    // setTimeout(() => {
    //     const mokstatus = randomdelay % 2 == 0 ? true : false
    //     log(`O envio para o numero: ${contato.numero} do contato: ${contato.id} retornou sucesso: ${mokstatus}`)
    //     new ContatoNumero({ id: contato.id, processando: false, sucesso: mokstatus }).save()
    //     if (mokstatus && config.atendimento.criar) {
    //         criarAtendimento(campanha, contato)
    //     }
    // }, randomdelay);
}

async function criarAtendimento(campanha, contato) {
    const status = config.atendimento.status
    contato.nome = ''
    if (contato.campos && typeof contato.campos === 'object' && contato.campos.hasOwnProperty(campanha.campo_nome)) {
        contato.nome = contato.campos[campanha.campo_nome]
    }

    const cliente = await Cliente.getOrCreate(contato.numero, contato.nome)
    const atendimento = await new Atendimento({
        queue: campanha.fila.name,
        queueTypeID: campanha.fila.queue_type,
        sentido: "Sainte",
        status: status,
        idCliente: cliente.id,
        fromID: cliente.numero,
        dataFim: status === 'Encerrado' ? new Date() : null
    }).save()
    log(`Atendimento id = ${atendimento.id} criado para contato cliente id = ${cliente.id}, nome: ${cliente.nome}`)
    return atendimento
}

async function run() {
    log('start')
    await upgrade.run()

    const campanhas = config.campanhas
    log('Vamos executar para campanhas: ', campanhas)
    runCampanhas(campanhas)
}

run()