const db = require('../util/db')

class Cliente extends db.orm.Model { }

/**
 * Usa a função find_cliente criada no Postgres para procurar o cliente pelo numero principal ou numero adicional.
 * @param {String} numero
 * @returns Array
 */
Cliente.BuscaClientePorNumero = async (numero) => {
    const ret = await Cliente.query.execute({
        text: 'select * from find_cliente($1)',
        values: [numero]
    });
    return ret[0] || undefined
};

/**
 * Retorna o cliente ou cadastra com numero e nome e retorna o cliente cadastrado
 * @param {string} numero 
 * @param {string} nome 
 * @returns 
 */
Cliente.getOrCreate = async (numero, nome) => {
    const cliente = await Cliente.BuscaClientePorNumero(numero)
    return cliente || await new Cliente({ numero, nome }).save()
};

Cliente.table = 'cliente';
Cliente.fields = {
    id: { type: 'integer', primary: true, updated: false, methods: true },
    numero: { type: 'string', methods: true },
    isBusiness: 'boolean',
    nome: { type: 'string' },
    documento: { type: 'string' },
    profileImage: 'string',
    idGrupo: 'integer',
    url: { type: `string` },
    email: { type: 'string', methods: true },
    facebookID: { type: 'string', methods: true }
}

module.exports = Cliente;