const config = require('../config.json')
const ContatoCampo = require('../model/CampanhaContatoCampo')
const { log } = require('../util/log')
const socketIO = require('socket.io-client')
var socket = null
var isConetado = false

exports.start = async () => {
    if (isConetado) return;

    console.info(`Inciando conexão em ${config.csa.gupshup.host} com ClientID = ${config.csa.gupshup.clientID}`);
    socket = socketIO(config.csa.gupshup.host)
    // socket.on('error_gupshup', (e) => {
    //     log(`[GUP-ERRO] error_gupshup`, e)
    // });

    socket.on('disconnect', (e) => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, DISCONECTED`, e)
        isConetado = false
    });

    socket.on('connect_timeout', (timeout) => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, TIMEOUT`, timeout)
        isConetado = false
    });
    socket.on('error', (error) => {
        log(`[GUP-ERRO] SOCKET ERRO, ERROR`, error)
        isConetado = false
    });
    socket.on('connect_error', (error) => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, CONNNECT_ERROR`, error)
        isConetado = false
    });
    socket.on('reconnecting', (attemptNumber) => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, RECONECTIONG`, attemptNumber)
        isConetado = false
    });
    socket.on('reconnect_failed', () => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, RECONNECT_FAIL`)
        isConetado = false
    });
    socket.on('unauthorized', (err) => {
        log(`[GUP-ERRO] SOCKET CONN_ERRO, NAO_AUTORIZADO`, err)
        isConetado = false
    });

    socket.on('connect', () => {
        console.info(`Conexão GUP aberta com ${config.csa.gupshup.host} e ClientID = ${config.csa.gupshup.clientID}`);

        socket.emit('Login', { clientID: config.csa.gupshup.clientID }, (ret) => {
            // Verifica se o login foi bem sucedido
            if (ret.error) {
                log(`[GUP-ERRO] Erro ao autenticar-se com o socket usando ClientID = ${cconfig.csa.gupshup.clientID}` + ret.error);
                isConetado = false
            } else {
                console.info(`Login efetuado com sucesso para CSA GUP usando ClientID = ${config.csa.gupshup.clientID}`);
                isConetado = true
            }
        });
    })
}

exports.mensagens_por_segundo = config.csa.gupshup.mensagens_por_segundo

exports.enviaComTemplate = async (campanha, contato) => {
    // Aqui entra a questão das variaveis do template
    // Se tiver que olhar as variaveis vou precisar buscar os campos do contato
    //console.log(campanha)
    //console.log(contato)
    let imageUrl = undefined
    const camposAux = []

    if (campanha.rota && typeof campanha.rota === 'object') {
        if (contato.campos && typeof contato.campos === 'object' && contato.campos.hasOwnProperty('imageUrl')) {
            imageUrl = JSON.parse(JSON.stringify(contato.campos['imageUrl']))
            delete contato.campos['imageUrl']
        }
        if (contato.campos && typeof contato.campos === 'object') {
            campanha.layoutsCampos.map(v => {
                if (contato.campos.hasOwnProperty(v.nome) && v.xcontact === 'variavelTemplate') camposAux.push(contato.campos[`${v.nome}`])
            })
        }
    }
    const msg_template = {
        apikey: campanha.fila.emailConfig.apikey,
        source: campanha.fila.emailConfig.source,
        appName: campanha.fila.emailConfig.appName,
        to: contato.numero,
        template: {
            id: campanha.rota.id,
            imageUrl: imageUrl,
            fields: camposAux
        }
    }
    // console.log('Iria enviar: ', msg_template)

    return new Promise((resolve, reject) => {
        let timeout = setTimeout(() => {
            reject(new Error('Timeout, o servidor não respondeu sua chamada'))
        },
            20 * 1000
        )
        socket.emit('newWhatsMsgGupshup', msg_template, (retorno) => {
            log(`A API oficial não oficial retornou erro = `, retorno.error)
            clearTimeout(timeout)
            if (retorno && retorno.error) {
                reject(new Error(retorno.message))
            }
            else {
                resolve(retorno || true)
            }
        })
    })
}